from netbox.views import generic

__all__ = (
    'PluginObjectListView',
    'PluginObjectView',
    'PluginObjectEditView',
    'PluginObjectDeleteView',
    'PluginObjectImportView',
    'PluginObjectBulkCreateView',
    'PluginObjectBulkEditView',
    'PluginObjectBulkDeleteView',
    'PluginObjectBulkRenameView',
    'PluginObjectBulkImportView'
)


class PluginObjectListView(generic.ObjectListView):
    template_name = 'netbox_plugin_extensions/generic/object_list.html'


class PluginObjectView(generic.ObjectView):
    template_name = 'netbox_plugin_extensions/generic/object.html'


class PluginObjectEditView(generic.ObjectEditView):
    template_name = 'netbox_plugin_extensions/generic/object_edit.html'


class PluginObjectDeleteView(generic.ObjectDeleteView):
    template_name = 'netbox_plugin_extensions/generic/object_delete.html'


class PluginObjectImportView(generic.ObjectImportView):
    template_name = 'netbox_plugin_extensions/generic/object_import.html'


class PluginObjectBulkImportView(generic.BulkImportView):
    template_name = 'netbox_plugin_extensions/generic/object_bulk_import.html'


class PluginObjectBulkCreateView(generic.BulkCreateView):
    pass


class PluginObjectBulkEditView(generic.BulkEditView):
    template_name = 'netbox_plugin_extensions/generic/object_bulk_edit.html'


class PluginObjectBulkDeleteView(generic.BulkDeleteView):
    template_name = 'netbox_plugin_extensions/generic/object_bulk_delete.html'


class PluginObjectBulkRenameView(generic.BulkRenameView):
    template_name = 'netbox_plugin_extensions/generic/object_bulk_rename.html'



from django import template
from django.apps import apps
from django.urls import NoReverseMatch, reverse

from extras.plugins import PluginConfig
from utilities.forms import TableConfigForm

register = template.Library()

__all__ = (
    'plugin_viewname',
    'plugin_validated_viewname',
)


def _resolve_namespace(instance):
    """
    Get the appropriate namespace for the app based on whether it is a Plugin or base application
    """

    app = apps.get_app_config(instance._meta.app_label)
    if isinstance(app, PluginConfig):
        return f'plugins:{app.label}'
    return f'{app.label}'


def get_returnurl(instance, parent=None):
    """
    Get an appropriate return url
    """
    if parent is None:
        return reverse(_get_plugin_viewname(instance, 'list'))
    else:
        return reverse(_get_plugin_viewname(parent), kwargs={'pk': parent.pk})


def _get_plugin_viewname(instance, action=None):
    """
    Return the appropriate viewname for adding, editing, viewing changelog or deleting an instance.
    """

    # Validate action
    if action is not None:
        assert action in ('add', 'edit', 'delete', 'list', 'changelog')
    app_label = _resolve_namespace(instance)
    if action is not None:
        viewname = f'{app_label}:{instance._meta.model_name}_{action}'
    else:
        viewname = f'{app_label}:{instance._meta.model_name}'
    return viewname


@register.filter()
def plugin_viewname(model, action):
    """
    Return the view name for the given model and action. Does not perform any validation.
    """
    namespace = _resolve_namespace(model)
    return f'{namespace}:{model._meta.model_name}_{action}'


@register.filter()
def plugin_validated_viewname(model, action):
    """
    Return the view name for the given model and action if valid, or None if invalid.
    """
    namespace = _resolve_namespace(model)
    viewname = f'{namespace}:{model._meta.model_name}_{action}'
    try:
        # Validate and return the view name. We don't return the actual URL yet because many of the templates
        # are written to pass a name to {% url %}.
        reverse(viewname)
        return viewname
    except NoReverseMatch:
        return None



try:
    from netbox.api.serializers import NestedGroupModelSerializer as OrganizationalModelSerializer
except ImportError:
    from netbox.api.serializers import OrganizationalModelSerializer

from netbox.api.serializers import ValidatedModelSerializer, WritableNestedSerializer, PrimaryModelSerializer,\
    NestedGroupModelSerializer, BulkOperationSerializer


class PluginValidatedModelSerializer(ValidatedModelSerializer):
    pass


#
# Nested serializers
#

class PluginWritableNestedSerializer(WritableNestedSerializer):
    pass


#
# Base model serializers
#

class PluginOrganizationalModelSerializer(OrganizationalModelSerializer):
    pass


class PluginPrimaryModelSerializer(PrimaryModelSerializer):
    pass


class PluginNestedGroupModelSerializer(NestedGroupModelSerializer):
    pass


class PluginBulkOperationSerializer(BulkOperationSerializer):
    pass



from django import template
from django.urls import reverse
from netbox_plugin_extensions.templatetags.plugin_helpers import get_returnurl

from extras.models import ExportTemplate
from utilities.utils import prepare_cloned_fields
from .plugin_helpers import _get_plugin_viewname

register = template.Library()


#
# Instance buttons
#

@register.inclusion_tag('buttons/clone.html')
def plugin_clone_button(instance):
    url = reverse(_get_plugin_viewname(instance, 'add'))

    # Populate cloned field values
    param_string = prepare_cloned_fields(instance)
    if param_string:
        url = f'{url}?{param_string}'

    return {
        'url': url,
    }


@register.inclusion_tag('buttons/edit.html')
def plugin_edit_button(instance):
    viewname = _get_plugin_viewname(instance, 'edit')
    url = reverse(viewname, kwargs={'pk': instance.pk})

    return {
        'url': url,
    }


@register.inclusion_tag('buttons/delete.html')
def plugin_delete_button(instance):
    viewname = _get_plugin_viewname(instance, 'delete')
    url = reverse(viewname, kwargs={'pk': instance.pk})

    return {
        'url': url,
    }


#
# List buttons
#

@register.inclusion_tag('buttons/add.html')
def plugin_add_button(instance):
    viewname = _get_plugin_viewname(instance, 'add')
    url = reverse(viewname)

    return {
        'add_url': url,
    }


@register.inclusion_tag('buttons/import.html')
def plugin_import_button(instance):
    viewname = _get_plugin_viewname(instance, 'import')
    url = reverse(viewname)

    return {
        'import_url': url,
    }


@register.inclusion_tag('buttons/export.html', takes_context=True)
def plugin_export_button(context, content_type=None):
    add_exporttemplate_link = None

    if content_type is not None:
        user = context['request'].user
        export_templates = ExportTemplate.objects.restrict(user, 'view').filter(content_type=content_type)
        if user.is_staff and user.has_perm('extras.add_exporttemplate'):
            add_exporttemplate_link = f"{reverse('extras:exporttemplate_add')}?content_type={content_type.pk}"
    else:
        export_templates = []

    return {
        'url_params': context['request'].GET,
        'export_templates': export_templates,
        'add_exporttemplate_link': add_exporttemplate_link,
    }

#
# Table buttons
#


@register.inclusion_tag('netbox_plugin_extensions/buttons/tr_edit.html')
def plugin_tr_edit_button(instance, extra=None, parent=None):
    viewname = _get_plugin_viewname(instance, 'edit')
    return_url = get_returnurl(instance, parent)
    url = reverse(viewname, kwargs={'pk': instance.pk})
    url = f'{url}?return_url={return_url}'

    if extra is not None:
        url = f'{url}{extra}'

    return {
        'url': url,
    }


@register.inclusion_tag('netbox_plugin_extensions/buttons/tr_delete.html')
def plugin_tr_delete_button(instance, extra=None, parent=None):
    viewname = _get_plugin_viewname(instance, 'delete')
    return_url = get_returnurl(instance, parent)
    url = reverse(viewname, kwargs={'pk': instance.pk})
    url = f'{url}?return_url={return_url}'

    if extra is not None:
        url = f'{url}{extra}'

    return {
        'url': url,
    }


@register.inclusion_tag('netbox_plugin_extensions/buttons/tr_changelog.html')
def plugin_tr_changelog_button(instance, extra=None, parent=None):
    viewname = _get_plugin_viewname(instance, 'changelog')
    url = reverse(viewname, kwargs={'pk': instance.pk})
    url = f'{url}'

    return {
        'url': url,
    }



from django.forms import BoundField
from django.urls import reverse

from utilities.forms.fields import DynamicModelChoiceMixin, DynamicModelChoiceField, DynamicModelMultipleChoiceField


__all__ = (
    'PluginDynamicModelChoiceField',
    'PluginDynamicModelMultipleChoiceField'
)


class PluginDynamicModelChoiceMixin(DynamicModelChoiceMixin):

    def get_bound_field(self, form, field_name):
        bound_field = BoundField(form, self, field_name)

        # Set initial value based on prescribed child fields (if not already set)
        if not self.initial and self.initial_params:
            filter_kwargs = {}
            for kwarg, child_field in self.initial_params.items():
                value = form.initial.get(child_field.lstrip('$'))
                if value:
                    filter_kwargs[kwarg] = value
            if filter_kwargs:
                self.initial = self.queryset.filter(**filter_kwargs).first()

        # Modify the QuerySet of the field before we return it. Limit choices to any data already bound: Options
        # will be populated on-demand via the APISelect widget.
        data = bound_field.value()
        if data:
            field_name = getattr(self, 'to_field_name') or 'pk'
            filter = self.filter(field_name=field_name)
            try:
                self.queryset = filter.filter(self.queryset, data)
            except (TypeError, ValueError):
                # Catch any error caused by invalid initial data passed from the user
                self.queryset = self.queryset.none()
        else:
            self.queryset = self.queryset.none()

        # Set the data URL on the APISelect widget (if not already set)
        widget = bound_field.field.widget
        if not widget.attrs.get('data-url'):
            app_label = self.queryset.model._meta.app_label
            model_name = self.queryset.model._meta.model_name
            data_url = reverse('plugins-api:{}-api:{}-list'.format(app_label, model_name))
            widget.attrs['data-url'] = data_url

        return bound_field


class PluginDynamicModelChoiceField(PluginDynamicModelChoiceMixin, DynamicModelChoiceField):
    pass


class PluginDynamicModelMultipleChoiceField(PluginDynamicModelChoiceMixin, DynamicModelMultipleChoiceField):
    pass



from netbox_plugin_extensions.tables.objectchange import PluginObjectChangeTable

from extras.models import ObjectChange
from extras.views import ObjectChangeListView
from netbox_plugin_extensions.views.generic import PluginObjectListView


#
# Change logging
#
class PluginObjectChangeListView(PluginObjectListView, ObjectChangeListView):
    table = PluginObjectChangeTable
    template_name = 'extras/objectchange_list.html'
    action_buttons = ('export',)



from netbox.api import ChoiceField, ContentTypeField, SerializedPKRelatedField


class PluginChoiceField(ChoiceField):
    pass


class PluginContentTypeField(ContentTypeField):
    pass


class PluginSerializedPKRelatedField(SerializedPKRelatedField):
    pass



from extras.models import ObjectChange
from netbox_plugin_extensions.tables import PluginButtonsColumn
from extras.tables import ObjectChangeTable


class PluginObjectChangeTable(ObjectChangeTable):

    actions = PluginButtonsColumn(ObjectChange)



from utilities.tables import ButtonsColumn


class PluginButtonsColumn(ButtonsColumn):
    """
    Render edit, delete, and changelog buttons for an object.

    :param model: Model class to use for calculating URL view names
    :param prepend_content: Additional template content to render in the column (optional)
    :param return_url_extra: String to append to the return URL (e.g. for specifying a tab) (optional)
    """

    template_code = """
    {{% load plugin_buttons %}}
    {{% if "changelog" in buttons %}}
        {{% plugin_tr_changelog_button record %}}
    {{% endif %}}
    {{% if "edit" in buttons and perms.{app_label}.change_{model_name} %}}
        {{% plugin_tr_edit_button record return_url_extra %}}
    {{% endif %}}
    {{% if "delete" in buttons and perms.{app_label}.delete_{model_name} %}}
        {{% plugin_tr_delete_button record return_url_extra %}}
    {{% endif %}}
    """



from extras.plugins import PluginConfig


try:
    from importlib.metadata import metadata
except ModuleNotFoundError:
    from importlib_metadata import metadata

plugin = metadata('netbox_plugin_extensions')


class NetboxPluginExtensions(PluginConfig):
    name = plugin.get('Name').replace('-', '_')
    verbose_name = plugin.get('Summary')
    description = plugin.get('Description')
    version = plugin.get('Version')
    author = plugin.get('Author')
    author_email = plugin.get('Author-email')
    base_url = 'netbox-plugin-extensions'
    min_version = '3.0'
    required_settings = []
    caching_config = {}
    default_settings = {}


config = NetboxPluginExtensions



from setuptools import find_packages, setup

setup(
    name='netbox-plugin-extensions',
    version='1.0.6',
    description='NetBox Plugin Extensions',
    long_description='Wrappers for Netbox Generic Objects',
    url='https://github.com/dansheps/netbox-plugin-extensions/',
    download_url='https://pypi.org/project/netbox-plugin-extensions/',
    author='Daniel Sheppard',
    author_email='dans@dansheps.com',
    maintainer='Daniel Sheppard',
    maintainer_email='dans@dansheps.com',
    license='All rights reserved',
    platform=[],
    keywords=['netbox', 'netbox-plugin'],
    classifiers=[
        'Framework :: Django',
        'Programming Language :: Python :: 3',
    ],
    install_requires=[
        'importlib',
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)


from netbox.models import PrimaryModel, OrganizationalModel, NestedGroupModel, ChangeLoggedModel, BigIDModel

__all__ = (
    'PluginBigIDModel',
    'PluginChangeLoggedModel',
    'PluginNestedGroupModel',
    'PluginOrganizationalModel',
    'PluginPrimaryModel',
)


class PluginBigIDModel(BigIDModel):

    class Meta:
        abstract = True


class PluginChangeLoggedModel(ChangeLoggedModel):

    class Meta:
        abstract = True


class PluginNestedGroupModel(NestedGroupModel):

    class Meta:
        abstract = True


class PluginOrganizationalModel(OrganizationalModel):

    class Meta:
        abstract = True


class PluginPrimaryModel(PrimaryModel):

    class Meta:
        abstract = True



from netbox.filtersets import BaseFilterSet, OrganizationalModelFilterSet, PrimaryModelFilterSet, \
    ChangeLoggedModelFilterSet


class PluginBaseFilterSet(BaseFilterSet):
    pass


class PluginChangeLoggedModelFilterSet(ChangeLoggedModelFilterSet):
    pass


class PluginPrimaryModelFilterSet(PrimaryModelFilterSet):
    pass


class PluginOrganizationalModelFilterSet(OrganizationalModelFilterSet):
    pass
